local L = LibStub("AceLocale-3.0"):GetLocale("AVRE", false)
local AVRE = AVRE
local mod = AVRE:NewModule("Options", "AceConsole-3.0")
local zones = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")



local newOrder
do
	local current = 0
	function newOrder()
		current = current + 1
		return current
	end
end

local function getRaidUnits(current)
	local t = { [""] = "" }
	
	if current and current ~= "" then t[current] = AVRE:Colorize(current, GRAY_FONT_COLOR) end
	
	if UnitInRaid("player") then
		local numRaid = GetNumRaidMembers()
		for i = 1, numRaid do
			local playerName = GetRaidRosterInfo(i)
			if playerName then
				t[playerName] = AVRE:Colorize(playerName, (RAID_CLASS_COLORS[({UnitClass(playerName)})[2]]))
			end
		end
	end
	
	return t
end

local newExpansion
local newInstance
local newFight
local a
local b
local c

function newExpansion(n, s)
	a = s
	return n
end

function newInstance(n, s)
	b = s
	return n
end

function newFight(n, s)
	c = s
	return n
end

function checkbox_spell(a, b, c, spellId, message, option)
	return {
		type = "toggle",
		name = GetSpellInfo(spellId),
		desc = format(message, GetSpellInfo(spellId)),
		get = function() return AVRE.db.profile.enc[a][b][c][option] end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c][option] = val end,
		order = newOrder(),
	}
end

function checkbox(a, b, c, name, option, desc)
	return {
		type = "toggle",
		name = name,
		desc = desc,
		get = function() return AVRE.db.profile.enc[a][b][c][option] end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c][option] = val end,
		order = newOrder(),
	}
end

function splitter()
	return {
		type = "header",
		name = "",
		order = newOrder(),
	}
end

function range1()
	return splitter()
end

function range2(a, b, c, extraDesc)
	return {
		type = "toggle",
		name = L["Enable range check"],
		desc = L["Turns on an AVR range check when the fight starts."].." "..extraDesc or "",
		get = function() return AVRE.db.profile.enc[a][b][c].range.enabled end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c].range.enabled = val end,
		order = newOrder(),
	}
end

function range3(a, b, c)
	return {
		order = newOrder(),
		type = "range",
		name = L["Range"],
		desc = L["The distance in yards for the AVR range check."],
		min = 1,
		max = 30,
		step = 1,
		get = function() return AVRE.db.profile.enc[a][b][c].range.range end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c].range.range = val end,
	}
end

function character_picker(a, b, c, name, option, desc)
	return {
		order = newOrder(),
		type = "select",
		name = name,
		desc = desc,
		values = function() return getRaidUnits(AVRE.db.profile.enc[a][b][c][option]) end,
		get = function() return AVRE.db.profile.enc[a][b][c][option] end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c][option] = val end,
	}
end



--[[
      OPTIONS ------------------------------------------------------------------------------------------------------------------
  ]]

local debugMsg = ""
--[===[@debug@
debugMsg = "    (running in debug mode)"
--@end-debug@]===]

AVRE.options = {
	name = "AVR Encounters         r|cff20ff20"..AVRE.MINOR_VERSION.."|r"..debugMsg,
	handler = VampArrow,
	type = "group",
	args = {		

		enabled = {
			type = "toggle",
			name = L["Enable AVR Encounters"],
			get = function() return AVRE.db.profile.enabled end,
			set = function(self, val)
				AVRE.db.profile.enabled = val
				if val then AVRE:Enable() else AVRE:Disable() end
			end,
			order = newOrder(),
		}, -- enabled
		
--------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------
-- WRATH OF THE LICH KING ------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------

		wotlk = {
			name = newExpansion(L["Wrath of the Lich King"], "wotlk"),
			type = "group",
			order = newOrder(),
			args = {
			
				enabled = {
					type = "toggle",
					name = L["Enable Expansion"],
					get = function() return AVRE.db.profile.enc.wotlk.enabled end,
					set = function(self, val) AVRE.db.profile.enc.wotlk.enabled = val end,
					order = newOrder(),
				}, -- enabled

--------------------------------------------------------------------------------------------------------------------
-- ICECROWN CITADEL ------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------

				icecrown = {
					name = newInstance(zones["Icecrown Citadel"], "icecrown"),
					type = "group",
					order = newOrder(),
					args = {
						
						enabled = {
							type = "toggle",
							name = L["Enable Raid Instance"],
							get = function() return AVRE.db.profile.enc.wotlk.icecrown.enabled end,
							set = function(self, val) AVRE.db.profile.enc.wotlk.icecrown.enabled = val end,
							order = newOrder(),
						}, -- enabled
						
						
-- LORD MARROWGAR --------------------------------------------------------------------------------------------------

						marrowgar = {
							name = newFight(L["Lord Marrowgar"], "marrowgar"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								spike = checkbox_spell(a,b,c, 69062, L["Places a circle under players with %s, along with a marker line toward them."], "spike"),
							},
						},
						
-- LADY DEATHWHISPER -----------------------------------------------------------------------------------------------

						deathwhisper = {
							name = newFight(L["Lady Deathwhisper"], "deathwhisper"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								spike = checkbox_spell(a,b,c, 71289, L["Places a circle under players with %s."], "dominate"),
								spirit = checkbox_spell(a,b,c, 71426, L["Places a circle under players with %s."], "spirit"),
							},
						},
						
-- FESTERGUT -------------------------------------------------------------------------------------------------------

						festergut = {
							name = newFight(L["Festergut"], "festergut"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								gasSpore = checkbox_spell(a,b,c, 69279, L["Places a circle under players with %s."], "gasSpore"),
								range1 = range1(),
								range2 = range2(a,b,c, L["(Heroic mode only.)"]),
								range3 = range3(a,b,c),
							},
						},
						
-- ROTFACE ---------------------------------------------------------------------------------------------------------

						rotface = {
							name = newFight(L["Rotface"], "rotface"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								explosion = checkbox_spell(a,b,c, 69839, L["Places a circle under all players during %s."], "explosion"),
								infection = checkbox_spell(a,b,c, 69674, L["Places a circle under players with %s."], "infection"),
								vile = checkbox_spell(a,b,c, 72272, L["Places a circle under players with %s."], "vile"),
								kiter = character_picker(a,b,c, L["Big Ooze Kiter"], "kiter", L["Choose a player in your raid who will be kiting the Big Oozes. They will have a small circle under them throughout the fight, and you'll get a marker toward that person when you get a Small Ooze."]),
								range1 = range1(),
								range2 = range2(a,b,c, L["(Heroic mode only.)"]),
								range3 = range3(a,b,c),
							},
						},
						
-- PUTRICIDE -------------------------------------------------------------------------------------------------------

						putricide = {
							name = newFight(L["Professor Putricide"], "putricide"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								malleable = checkbox_spell(a,b,c, 72615, L["Places a circle under players with %s."].." "..L["Only works for one of the players."], "malleable"),
								malleableAll = checkbox(a,b,c, format(L["%s (All)"], GetSpellInfo(72615)), "malleableAll", format(L["Places a circle under all players during %s."], GetSpellInfo(72615))),
								plague = checkbox_spell(a,b,c, 72855, L["Places a circle under players with %s."], "plague"),
								sick = checkbox(a,b,c, GetSpellInfo(70953), "sick", format(L["When you are afflicted with %s, you will see a circle under all players who do not have %s. These are valid players to pass %s to."], GetSpellInfo(72855), GetSpellInfo(70953), GetSpellInfo(72855))),
							},
						},
						
-- BLOOD PRINCE COUNCIL --------------------------------------------------------------------------------------------

						bloodprince = {
							name = newFight(L["Blood Prince Council"], "bloodprince"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								flame = checkbox_spell(a,b,c, 72040, L["Places a circle under players with %s."], "flame"),
								vortex = checkbox_spell(a,b,c, 72037, L["Places a circle under players with %s."], "vortex"),
							},
						},
						
-- SINDRAGOSA ------------------------------------------------------------------------------------------------------

						sindragosa = {
							name = newFight(L["Sindragosa"], "sindragosa"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								beacon = checkbox_spell(a,b,c, 70126, L["Places a circle under players with %s."], "beacon"),
								unchained = checkbox_spell(a,b,c, 69762, L["Places a circle under players with %s."].." "..L["(Heroic mode only.)"], "unchained"),
							},
						},
						
-- LICH KING -------------------------------------------------------------------------------------------------------

						lichking = {
							name = newFight(L["The Lich King"], "lichking"),
							type = "group",
							order = newOrder(),
							args = {
								enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
								split = splitter(),
								defile = checkbox_spell(a,b,c, 72762, L["Places a circle under players with %s."], "defile"),
								shadowtrap = checkbox_spell(a,b,c, 73539, L["Places a circle under players with %s."], "shadowtrap"),
							},
						},
						
						
					},
				},
				
			},
		},

	}
}



--[[
      INITIALIZATION -----------------------------------------------------------------------------------------------------------
  ]]

function mod:OnInitialize()
	-- Register options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AVRE", AVRE.options)
	AVRE.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AVRE", "AVR Encounters")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AVRE-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(AVRE.db))
	AVRE.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AVRE-Profiles", "Profiles", "AVR Encounters")
	AVRE.modules.options.loaded = true
end


function mod:OnEnable()
--[===[@debug@
	self:Print(L["Loaded"])
--@end-debug@]===]
end
